@@ -31,13 +31,6 @@ ets <- function(y, model="ZZZ", damped=NULL,
     if(nmse < 1 | nmse > 10)
         stop("nmse out of range")
     m <- frequency(y)
-    if(m>24)
-        stop("Frequency too high")
-    else if(m<1)
-    {
-        warning("I can't handle data with frequency less than 1. Seasonality will be ignored.")
-        m <- 1
-    }
 
     if(sum((upper-lower)>0)<4)
         stop("Lower limits must be less than upper limits")
@@ -70,19 +63,28 @@ ets <- function(y, model="ZZZ", damped=NULL,
     if(!is.element(seasontype,c("N","A","M","Z")))
         stop("Invalid season type")
 
-    if(m <= 1)
+    if(m < 1)
     {
-        if(seasontype=="A" | seasontype=="M")
-            stop("Nonseasonal data")
-        else
-            substr(model,3,3) <- seasontype <- "N"
+      warning("I can't handle data with frequency less than 1. Seasonality will be ignored.")
+      m <- 1
+    }
+    if(m == 1)
+    {
+      if(seasontype=="A" | seasontype=="M")
+        stop("Nonseasonal data")
+      else
+        substr(model,3,3) <- seasontype <- "N"
+    }
+    if(m > 24)
+    {
+      if(is.element(seasontype,c("A","M")))
+        stop("Frequency too high")
+      else if(seasontype=="Z")
+      {
+        warning("I can't handle data with frequency greater than 24. Seasonality will be ignored. Try stlf() if you need seasonal forecasts.")
+        substr(model,3,3) <- seasontype <- "N"
+      }
     }
-#    else if(length(y) < 3*m & (seasontype!="N"))
-#    {
-#        if(seasontype != "Z")
-#            warning("Insufficient data to fit a seasonal model")
-#        substr(model,3,3) <- seasontype <- "N"
-#    }
 
     # Check inputs
     if(restrict)
